@@ -120,7 +120,9 @@ namespace gnote {
     Gtk::Application::on_command_line(command_line);
     int argc = 0;
     char **argv = command_line->get_arguments(argc);
-    cmd_line.parse(argc, argv);
+    GnoteCommandLine passed_cmd_line;
+    GnoteCommandLine &cmdline = m_manager ? passed_cmd_line : cmd_line;
+    cmdline.parse(argc, argv);
     if(!m_manager) {
       common_init();
       Glib::RefPtr<Gio::Settings> settings = Preferences::obj()
@@ -129,26 +131,18 @@ namespace gnote {
         .connect(sigc::mem_fun(*this, &Gnote::on_setting_changed));
       register_object();
     }
-    else if(cmd_line.needs_execute()) {
-      cmd_line.execute();
+    else if(cmdline.needs_execute()) {
+      cmdline.execute();
     }
-    else {
+    else if(!cmdline.background()) {
       new_main_window().present();
     }
 
+    g_strfreev(argv);
     return 0;
   }
 
 
-  void Gnote::on_window_removed(Gtk::Window *window)
-  {
-    // Do not remove last window if background or status icon mode
-    if(windowed() || get_windows().size() > 1) {
-      Gtk::Application::on_window_removed(window);
-    }
-  }
-
-
   void Gnote::common_init()
   {
     std::string note_path = get_note_path(cmd_line.note_path());
@@ -186,17 +180,19 @@ namespace gnote {
       else {
         // If Gnote is already running, open the search window
         // so the user gets some sort of feedback when they
-        // attempt to run Gnote again.
-        Glib::RefPtr<RemoteControlClient> remote;
-        try {
-          remote = RemoteControlProxy::get_instance();
-          DBG_ASSERT(remote, "remote is NULL, something is wrong");
-          if(remote) {
-            remote->DisplaySearch();
+        // attempt to run Gnote again, except when --background is passed
+        if(!m_is_background) {
+          Glib::RefPtr<RemoteControlClient> remote;
+          try {
+            remote = RemoteControlProxy::get_instance();
+            DBG_ASSERT(remote, "remote is NULL, something is wrong");
+            if(remote) {
+              remote->DisplaySearch();
+            }
+          } 
+          catch (...)
+          {
           }
-        } 
-        catch (...)
-        {
         }
 
         ERR_OUT(_("Gnote is already running.  Exiting..."));
@@ -213,8 +209,8 @@ namespace gnote {
       start_tray_icon();
     }
     else if(m_is_background) {
-      // Create Search All Notes window as we need it present for application to run
-      new_main_window();
+      // do not exit when all windows are closed
+      hold();
     }
     else {
       get_main_window().present();
@@ -432,10 +428,6 @@ namespace gnote {
   void Gnote::on_main_window_closed(Gtk::Window *win)
   {
     delete win;
-    // if background mode, we need to have a window, to prevent quit
-    if(m_is_background && !Gtk::Window::list_toplevels().size()) {
-      new_main_window();
-    }
   }
 
   MainWindow & Gnote::get_window_for_note()
@@ -566,7 +558,7 @@ namespace gnote {
     , m_open_start_here(false)
     , m_highlight_search(NULL)
   {
-    static const GOptionEntry entries[] =
+    const GOptionEntry entries[] =
       {
         { "background", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &m_background, _("Run Gnote in background."), NULL },
         { "note-path", 0, 0, G_OPTION_ARG_STRING, &m_note_path, _("Specify the path of the directory containing the notes."), _("path") },
